#----------------------------------------------------------------------------------------------------
#Author: Warren Jacobus
#        3065044
#        Univeristy of the Western Cape
#----------------------------------------------------------------------------------------------------
import compute
import display
import draw_graph
import ast
import graph_to_json_graph
import algorithms
from graph import DiGraph
from os import system
system("mkdir graphs")
system("mkdir graphs/text")
system("mkdir graphs/images")
system("clear")
#----------------------------------------------------------------------------------------------------
#Description (demo.py):
#           Computes link margins, free space loss, distances, feasible links using
#           imported scripts above.
#Inputs:
#           None.
#Outputs:
#           prints the tables of the link margins, free space loss, distances, and feasible links
#           as well as the graph visualizations of the mesh network and the mst's thereof.
#Caveats:
#           Described below.
#----------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------
#writeGraphToFile:
#            Writes the mesh network to a text file.
#Inputs:
#            A file name and well as the list of all feasible links i.e. the mesh network
#Outputs:
#            None.
#Caveats:
#            There are no limitations to date.
#----------------------------------------------------------------------------------------------------
def writeGraphToFile(filename,links): # write mesh to file
   f = open(filename,'w')
   for link in links:
      f.write(str(link) + "\n")
   f.close()
#----------------------------------------------------------------------------------------------------
#main:
#            Computes link margins, free space loss, distances, feasible links.
#Inputs:
#            None.
#Outputs:
#            Prints the tables of the link margins, free space loss, distances, and feasible links
#            as well as the graph visualizations of the mesh network and the mst's thereof.
#Caveats:
#            There are no limitations to date.
#----------------------------------------------------------------------------------------------------
def run(locations_file,specs_file):
   # get gps data from locations file
   f = open('gps_cords/locations.txt','r')
   locations = [(i.strip("\n")).split(" ") for i in f.readlines()]

   places_name = [i[0] for i in locations]
   places = [ast.literal_eval(i[1]) for i in locations]
   f.close()

   # compute distance between each node
   distances = compute.Distances(places,places_name)
   display.Distances(distances)

   #costs = [] # used as weights for prim and kruskal algorithm
   #for i in xrange(1,len(distances)):
   #   for j in xrange(i+1,len(distances)):
   #      costs.append((distances[i][0],distances[0][j],round(distances[i][j],2)))
   #costs.sort()

   # compute free space loss between each node
   lfs = compute.FreeSpaceLoss(distances,places,places_name)
   display.FreeSpaceLoss(lfs)

   # compute link margins between each node
   link_margins = compute.LinkMargins(lfs,places,places_name)
   display.LinkMargins(link_margins)

   # compute feasible links
   feasible = compute.FeasibleLinks(link_margins)
   display.Feasible(feasible)

   # write mesh to file
   writeGraphToFile("graphs/text/graph.txt",feasible)

   graph = [(place[0],place[1]) for place in feasible]
   labels = feasible[:]
   draw_graph.draw_graph(graph, labels)

   graph_to_json_graph.go()

   # Load the graph
   G = DiGraph("test")
   print 


   # Get the painting object and set its properties.
   paint = G.painter()
   paint.set_source_sink("UWC", "Kuils")
   #paint.set_rank_same(['C', 'D', 'F'])
   #paint.set_rank_same(['E', 'G', 'H'])
    
   # Generate the graph using the painter we configured.
   G.export(False, paint)
    
   # Get 10 shortest paths from the graph.
   #k = int(raw_input("Please enter the number of shortest paths you want: "))
   #src = raw_input("Please enter the source node you would like to start from: ") 
   #dest = raw_input("Please enter the destination node: ")
   k = len(locations) + 1
   items = algorithms.ksp_yen(G, "UWC", "Kuils", k)

   print "The %d shortest paths, from UWC to Kuilsriver_saps, in the mesh network is:"%k
   for path in items:
      print "Cost:%s\t%s" % (path['cost'], "->".join(path['path']))
   
   print
   print       
   return 0

#----------------------------------------------------------------------------------------------------
#END
#----------------------------------------------------------------------------------------------------
