#----------------------------------------------------------------------------------------------------
#Author: Warren Jacobus
#        3065044
#        Univeristy of the Western Cape
#----------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------
#Description (image_view.py):
#           Network tool that enables the planning of wireless networks.
#           Makes use of Kruskal,Prim, and Yen's algorithm.
#Inputs:
#           Gps cordinates of nodes and devices specification files
#Outputs:
#           pdf images of the mesh graph of the nodes.
#           two minimum spanning trees of the graph generated from prim and kruskal's algorithm.
#           the k-least-shortest-paths in the graph generated from Yen's algorithm
#Caveats:
#           Yen's algorithm does not have a threshold on which k produces sensible k-least-shortest-paths.
#           i.e., for some k, cycles will seem to be present.
#           This needs to be improved upon.
#           
#           GUI is not that inviting, it needs a lot of attention.
#       
#           2 Files are used as input. Improve to dynamically change parameters on GUI.
#----------------------------------------------------------------------------------------------------


import compute
import demo
import sys
import wx
import wx.richtext as rt
import wx.lib.imagebrowser as imagebrowser

class DisplayGraph(wx.Frame):
    def __init__(self,bitmap,window_name):
        wx.Frame.__init__(self, None, title=window_name.split("/")[-1],size=(800,640))
        panel = wx.Panel(self)

        self.bmp = wx.StaticBitmap(self)

        vsizer = wx.BoxSizer(wx.VERTICAL)
        hsizer = wx.BoxSizer(wx.HORIZONTAL)

        vsizer.Add(self.bmp, 0, wx.ALIGN_CENTER)
        hsizer.AddStretchSpacer()
        hsizer.Add(vsizer, 0, wx.ALIGN_CENTER)
        hsizer.AddStretchSpacer()
        self.SetSizer(hsizer)

        self.bmp.SetBitmap(bitmap)
        self.Layout()
        self.bmp.Refresh()

class RedirectText(object):
    def __init__(self,aWxTextCtrl):
        self.out=aWxTextCtrl

    def write(self,string):
        self.out.WriteText(string)

class ImageDialogApp(wx.App):
    def OnInit(self):
        self.frame = ImageDialogFrame(None,title="NetworkTool")
        self.frame.SetSize((800,640))
        self.frame.Show()
        self.frame.Center()
        return True

class ImageDialogFrame(wx.Frame):
    def __init__(self, *args, **kwargs):
        super(ImageDialogFrame, self).__init__(*args,**kwargs)

        # Attributes
        self.panel = ImageDialogPanel(self)

class ImageDialogPanel(wx.Panel):
    def __init__(self, *args, **kwargs):
        super(ImageDialogPanel, self).__init__(*args,**kwargs)

        panel = wx.Panel(self)
        panel.SetBackgroundColour('#ededed')

        vBox = wx.BoxSizer(wx.VERTICAL)

        hBox2 = wx.BoxSizer(wx.HORIZONTAL)
        hBox2.Add(wx.StaticText(panel, label=""))
        vBox.Add(hBox2, flag=wx.LEFT, border=10)
        vBox.Add((-1,5))

        hBox3 = wx.BoxSizer(wx.HORIZONTAL)
        #self.urlFld = wx.TextCtrl(panel, style=wx.TE_MULTILINE|wx.TE_READONLY, size=(790,350))
        self.urlFld = rt.RichTextCtrl(panel,style=wx.TE_MULTILINE|wx.TE_READONLY,size=(790,350))
        hBox3.Add(self.urlFld, 1, flag=wx.EXPAND)
        vBox.Add(hBox3, flag=wx.EXPAND | wx.LEFT | wx.RIGHT, border=10)

        panel.SetSizer(vBox)

        # Redirect text here
        redir = RedirectText(self.urlFld)
        sys.stdout = redir

        # Attributes
        self.dirname = ''
        self.lastpath = None
        self.specs_filename = None
        self.gps_cords_filename = None
        self.state = False
        #self.bmp = wx.StaticBitmap(self)
        #self.btn = wx.Button(self, label="Choose Image (none)")
        self.specs_btn = wx.Button(self,label='1-Choose specs file (none)')
        self.gps_cords_btn = wx.Button(self,label='2-Choose gps file (none)')
        self.run_btn = wx.Button(self,label='3-Run')
        self.view_graph_btn = wx.Button(self,label='4-View graph')
        #self.prim_btn = wx.Button(self,label='5-Display primm')
        self.exit_btn = wx.Button(self,label='Exit')

        # Layout
        vsizer = wx.BoxSizer(wx.VERTICAL)
        hsizer = wx.BoxSizer(wx.HORIZONTAL)

        #vsizer.Add(self.bmp, 0, wx.ALIGN_RIGHT)
        #vsizer.AddSpacer((5, 5))
        vsizer.Add(panel,0,wx.ALIGN_LEFT)
        vsizer.AddSpacer((5,5))
        vsizer.Add(self.specs_btn, 0, wx.ALIGN_CENTER)
        vsizer.AddSpacer((5,5))
        vsizer.Add(self.gps_cords_btn, 0, wx.ALIGN_CENTER)
        vsizer.AddSpacer((5,5))
        vsizer.Add(self.run_btn, 0, wx.ALIGN_CENTER)
        vsizer.AddSpacer((5,5))
        vsizer.Add(self.view_graph_btn, 0, wx.ALIGN_CENTER)
        #vsizer.AddSpacer((5,5))
        #vsizer.Add(self.prim_btn, 0, wx.ALIGN_CENTER)
        vsizer.AddSpacer((5,5))
        vsizer.Add(self.exit_btn, 0, wx.ALIGN_CENTER)
        hsizer.AddStretchSpacer()
        hsizer.Add(vsizer, 0, wx.ALIGN_CENTER)
        hsizer.AddStretchSpacer()
        self.SetSizer(hsizer)

        # Event Handlers
        self.Bind(wx.EVT_BUTTON, self.OnShowDialogSpecs, self.specs_btn)
        self.Bind(wx.EVT_BUTTON, self.OnShowDialogGps, self.gps_cords_btn)
        self.Bind(wx.EVT_BUTTON, self.Run, self.run_btn)
        #self.Bind(wx.EVT_BUTTON, self.displayPrim, self.prim_btn)
        self.Bind(wx.EVT_BUTTON, self.ViewGraph, self.view_graph_btn)
        self.Bind(wx.EVT_BUTTON, self.ExitApp, self.exit_btn)

    def OnShowDialogSpecs(self, event):
        with wx.FileDialog(self, "Choose a file to open", self.dirname,"", "*.*", wx.OPEN) as dlg:
            if dlg.ShowModal() == wx.ID_OK:
                directory, filename = dlg.GetDirectory(), dlg.GetFilename()
                self.urlFld.LoadFile('/'.join((directory, filename)))
                self.specs_filename = str(directory)+"/"+str(filename)
                self.state = True
        if self.state:
            self.specs_btn.SetLabel("1-Choose specs file (done)")
            self.state = False

    def OnShowDialogGps(self, event):
        with wx.FileDialog(self, "Choose a file to open", self.dirname,"", "*.*", wx.OPEN) as dlg:
            if dlg.ShowModal() == wx.ID_OK:
                directory, filename = dlg.GetDirectory(), dlg.GetFilename()
                self.urlFld.LoadFile('/'.join((directory, filename)))
                self.gps_cords_filename = str(directory)+'/'+str(filename)
                self.state = True
        if self.state:
            self.gps_cords_btn.SetLabel("2-Choose gps file (done)")
            self.state = False

    def Run(self,event):
        if self.specs_filename != None and self.gps_cords_filename != None:
            
            demo.run(self.specs_filename,self.gps_cords_filename)
            self.specs_filename = None
            self.gps_cords_filename = None
            self.gps_cords_btn.SetLabel("2-Choose gps file (none)")
            self.specs_btn.SetLabel("1-Choose specs file (none)")

    def ViewGraph(self,event):
        # Create the dialog with the path cached
        # from the last time it was opened
        dlg = imagebrowser.ImageDialog(self, self.lastpath)
        if dlg.ShowModal() == wx.ID_OK:
            # Save the last used path
            self.lastpath = dlg.GetDirectory()
            imgpath = dlg.GetFile()
            bitmap = wx.Bitmap(imgpath)
            if bitmap.IsOk():
                frame = DisplayGraph(bitmap,str(imgpath))
                frame.Show()

    #def displayPrim(self,event):
        # Create the dialog with the path cached
        # from the last time it was opened
        #dlg = imagebrowser.ImageDialog(self, self.lastpath)
        #if dlg.ShowModal() == wx.ID_OK:
            # Save the last used path
            #self.lastpath = dlg.GetDirectory()
            #imgpath = dlg.GetFile()
            #bitmap = wx.Bitmap(imgpath)
            #if bitmap.IsOk():
                #frame = DisplayGraph(bitmap,'Primm')
                #frame.Show()

    def ExitApp(self,event):
        sys.exit(0)
if __name__ == '__main__':
    app = ImageDialogApp(False)
    app.MainLoop()
